/**
 * @file Resolvers des utilisateurs
 * @author ofacklam
 */

import {Group, SimpleGroup, MetaGroup} from './groups';
import {Question} from './messages';
import {User as LDAP_User} from '../../ldap/export/user';
import {Tools, GroupSet} from '../connectors/tools';

export class User {

    /**
     * @memberof GraphQL
     * @class User
     * @summary Resolvers des utilisateurs
     * @classdesc Une classe représentant le type User du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} uid - Identifiant du user, supposé valide.
     * @rights connectedOrOnplatal
     */
    constructor(uid: string) {
        this.uid = uid;
        this.m_dataLoaded = false;
    }

    /**
     * @memberof GraphQL.User#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce user dans le LDAP, si ce n'est pas déja fait.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<void> {
        if(!this.m_dataLoaded) {
            let data = await LDAP_User.peek(this.uid);

            this.m_givenName = data.givenName;
            this.m_lastName = data.lastName;
            this.m_nickname = data.nickname;
            this.m_nationality = data.nationality;
            this.m_birthdate = data.birthdate;
            this.m_promotion = data.promotion;

            this.m_mail = data.mail;
            this.m_phone = data.phone;
            this.m_addresses = [data.address];

            this.m_memberOf = data.groups;
            //this.m_speakerOf = data.groupsIsSpeaker;
            this.m_adminOf = data.groupsIsAdmin;
            //this.m_likes = data.likes;

            this.m_dataLoaded = true;
        }
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete au LDAP,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_dataLoaded: boolean

    protected m_givenName: string
    protected m_lastName: string
    protected m_nickname: string
    protected m_nationality: string
    protected m_birthdate: string
    protected m_promotion: string

    protected m_mail: string
    protected m_phone: string
    protected m_addresses: string[]

    protected m_memberOf: string[]
    protected m_speakerOf: string[]
    protected m_adminOf: string[]
    protected m_likes: string[]

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    /** @rights connectedOrOnplatal */
    uid: string;

    /**
     * @memberof GraphQL.User#
     * @function givenName
     * @summary Renvoie le prénom
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async givenName(args, context, info): Promise<string> {
        await this.fetchData();
        return this.m_givenName;
    }

    /**
     * @memberof GraphQL.User#
     * @function lastName
     * @summary Renvoie le nom de famille
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async lastName(args, context, info): Promise<string> {
        await this.fetchData();
        return this.m_lastName;
    }

    /**
     * @memberof GraphQL.User#
     * @function nickname
     * @summary Renvoie le surnom
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async nickname(args, context, info): Promise<string> {
        await this.fetchData();
        return this.m_nickname;
    }

    /**
     * @memberof GraphQL.User#
     * @function nationality
     * @summary Renvoie la nationalité
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async nationality(args, context, info): Promise<string> {
        await this.fetchData();
        return this.m_nationality;
    }

    /**
     * @memberof GraphQL.User#
     * @function birthdate
     * @summary Renvoie la date de naissance
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async birthdate(args, context, info): Promise<string> {
        await this.fetchData();
        return this.m_birthdate;
    }

    /**
     * @memberof GraphQL.User#
     * @function promotion
     * @summary Renvoie la promotion
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async promotion(args, context, info): Promise<string> {
        await this.fetchData();
        return this.m_promotion;
    }

    /**
     * @memberof GraphQL.User#
     * @function mail
     * @summary Renvoie l'adresse mail
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async mail(args, context, info): Promise<string> {
        await this.fetchData();
        return this.m_mail;
    }

    /**
     * @memberof GraphQL.User#
     * @function phone
     * @summary Renvoie le numéro de téléphone
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async phone(args, context, info): Promise<string> {
        await this.fetchData();
        return this.m_phone;
    }

    /**
     * @memberof GraphQL.User#
     * @function addresses
     * @summary Renvoie les adresses
     * @return {Promise(string[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async addresses(args, context, info): Promise<string[]> {
        await this.fetchData();
        return this.m_addresses;
    }

    /**
     * @memberof GraphQL.User#
     * @function memberOf
     * @summary Renvoie les groupes dont il est membre
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async memberOf(args, context, info): Promise<Group[]> {
        await this.fetchData();
        let simple = new GroupSet(this.m_memberOf);
        let meta = await Tools.metaGroupsOfGroups(simple);

        let all: Group[];

        for(let gid of simple) {
            all.push(new SimpleGroup(gid));
        }
        for(let gid of meta) {
            all.push(new MetaGroup(gid));
        }

        return all;
    }

    /**
     * @memberof GraphQL.User#
     * @function speakerOf
     * @summary Renvoie les groupes dont il est speaker
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async speakerOf(args, context, info): Promise<Group[]> {
        await this.fetchData();
        let speaker = new GroupSet(this.m_speakerOf);
        let admin = new GroupSet(this.m_adminOf);
        let meta = await Tools.metaGroupsOfGroups(admin);

        let all: Group[];

        for (let gid of speaker) {
            all.push(new SimpleGroup(gid));
        }
        for (let gid of meta) {
            all.push(new MetaGroup(gid));
        }

        return all;
    }

    /**
     * @memberof GraphQL.User#
     * @function adminOf
     * @summary Renvoie les groupes dont il est admin
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async adminOf(args, context, info): Promise<Group[]> {
        await this.fetchData();
        let simple = new GroupSet(this.m_adminOf);
        let meta = await Tools.metaGroupsOfGroups(simple);

        let all: Group[];

        for (let gid of simple) {
            all.push(new SimpleGroup(gid));
        }
        for (let gid of meta) {
            all.push(new MetaGroup(gid));
        }

        return all;
    }

    /**
     * @memberof GraphQL.User#
     * @function likes
     * @summary Renvoie les groupes dont il est sympathisant
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async likes(args, context, info): Promise<Group[]> {
        await this.fetchData();
        let simple = new GroupSet(this.m_likes);
        let meta = await Tools.metaGroupsOfGroups(simple);

        let all: Group[];

        for (let gid of simple) {
            all.push(new SimpleGroup(gid));
        }
        for (let gid of meta) {
            all.push(new MetaGroup(gid));
        }

        return all;
    }

    /**
     * @memberof GraphQL.User#
     * @function questionsFromUser
     * @summary Renvoie les questions que le user a posées
     * @return {Promise(Question[])}
     * @rights viewer of recipient group
     * @async
     */
    async questionsFromUser(args, context, info): Promise<Question[]> {
        throw "Not implemented";
    }
 }